/*
 * $Id: main.c,v 1.6 2002/09/27 14:56:09 rlonn Exp $
 * $Source: /cvsroot/tptest/apps/unix/client/main.c,v $
 *
 * TPTEST 3.0 (C) Copyright II-Stiftelsen 2002
 *
 * main.c - simple client/server TPTEST application
 *
 * Written by
 *  Ragnar Lnn <prl@gatorhole.com>
 *
 * This file is part of the TPTEST system.
 * See the file LICENSE for copyright notice.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by       
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.       
 *
 * You should have received a copy of the GNU General Public License       
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 * 02111-1307  USA.
 *
 */


#include <stdio.h>
#ifdef UNIX
#include <unistd.h>
#endif
#include "tpengine.h"
#include "tpcommon.h"
#include "tpclient.h"
#include "tpio.h"

void do_test(TPEngine *, int);
void ReportStats(TPEngine *);

int main(int ac, char **av) {

	TPEngine *engp;
	int newmode, selectedMode;

	if (ac < 6) {
		if (ac < 3) {
			printf("Usage: %s [s/r/d/S/R/D/L]\n", av[0]);
			printf("\n");
			printf(" UDP test:  s/r/d [pktsize] [npackets] [time] [maxtime] [serverIP] [port]\n");
			printf(" TCP test:  S/R   [tcpbytes] [maxtime] [serverIP] [port]\n");
			printf(" Server:    D [local TCP control port]\n");
			printf(" Listsrv:   L [masterserver] [port]\n");
			printf(" Auto Send: As [serverIP] [port]\n");
			printf(" Auto Recv: Ar [serverIP] [port]\n");
			printf(" Auto:      A [serverIP] [port]\n");
			printf("\n");
			exit(1);
		}
	}

	printf("Initializing...\n");

	engp = CreateContext();
	if (engp == NULL) {
		printf("Error creating engine context\n");
		exit(1);
	}

	switch (av[1][0]) {
	case 's':
		selectedMode = CLM_UDP_SEND;
		break;
	case 'r':
		selectedMode = CLM_UDP_RECV;
		break;
	case 'd':
		selectedMode = CLM_UDP_FDX;
		break;
	case 'S':
		selectedMode = CLM_TCP_SEND;
		break;
	case 'R':
		selectedMode = CLM_TCP_RECV;
		break;
	case 'D':
		engp->myTCPControlPort = (USHORT)(atoi(av[2]));
		do_test(engp, 1);
		exit(0);
	case 'L':
		engp->hostCtrlPort = (USHORT)(atoi(av[3]));
		selectedMode = CLM_QUERY_MASTER;
		break;
	case 'A':
		switch (av[1][1]) {
		case 's':
			selectedMode = CLM_AUTO_SEND;
			break;
		case 'r':
			selectedMode = CLM_AUTO_RECV;
			break;
		default:
			selectedMode = CLM_AUTO;
			break;
		}
		break;
	default:
		printf("Unknown mode \"%c\"\n", av[1][0]);
		exit(1);
	}

	switch (selectedMode) {
	case CLM_UDP_SEND:
	case CLM_UDP_RECV:
	case CLM_UDP_FDX:
		engp->packetSize = atoi(av[2]);
		engp->nPackets = atoi(av[3]);
		engp->sessionTime = atoi(av[4]);
		engp->sessionMaxTime = atoi(av[5]);
		strcpy(engp->hostName, av[6]);
		engp->hostCtrlPort = (USHORT)(atoi(av[7]));
		break;
	case CLM_TCP_SEND:
	case CLM_TCP_RECV:
		engp->tcpBytes = atoi(av[2]);
		engp->sessionMaxTime = atoi(av[3]);
		strcpy(engp->hostName, av[4]);
		engp->hostCtrlPort = (USHORT)(atoi(av[5]));
		break;
	case CLM_QUERY_MASTER:
		strcpy(engp->hostName, av[2]);
		break;
	case CLM_AUTO:
	case CLM_AUTO_SEND:
	case CLM_AUTO_RECV:
		strcpy(engp->hostName, av[2]);
		engp->hostCtrlPort = (USHORT)(atoi(av[3]));
		break;
	}

	engp->hostIP.s_addr = inet_addr(engp->hostName);
	if (engp->hostIP.s_addr == INADDR_NONE) {
		printf("Invalid server address: %s (use the IP-number)\n", engp->hostName);
		exit(1);
	}

	newmode = AdvanceTest(engp, selectedMode, CLM_NONE, 0);
	while (newmode != CLM_NONE) {
		printf("Running test: %d\n", newmode);
		engp->tpMode = newmode;
		do_test(engp, 0);
		newmode = AdvanceTest(engp, selectedMode, newmode, engp->failCode);
	}

	if (selectedMode != CLM_QUERY_MASTER) {
		printf("Best TCP Send Rate: %s\n", Int32ToString((UINT32)(engp->bestTCPSendRate*8)));
		printf("Best TCP Recv Rate: %s\n", Int32ToString((UINT32)(engp->bestTCPRecvRate*8)));
		printf("Best UDP Send Rate: %s\n", Int32ToString((UINT32)(engp->bestUDPSendRate*8)));
		printf("Best UDP Recv Rate: %s\n", Int32ToString((UINT32)(engp->bestUDPRecvRate*8)));
	}
	return 0;
}



void do_test(TPEngine *engp, int doserver) {
	int iter = 0;
	int state = -1;

	if (doserver == 1) {
		for (;;) {
			if (StartServerContext(engp) != 0) {
				printf("StartServerContext() failed\n");
				exit(1);
			}
	
			while (engp->state != SSM_FAILED && engp->state != SSM_COMPLETE) {
				if (engp->state != state) {
					state = engp->state;
					printf("Server state change: new state = %d\n", state);
				}
				RunServerContext(engp);
				if (iter++ % 20 == 0) {
#ifdef UNIX
					usleep(30000);
#endif
#ifdef __WIN32
					Sleep(30);
#endif
				}
//				if (engp->callMeAgain != 0) 
//					Sleep(engp->callMeAgain / 1000);
			}
			ReportStats(engp);
#ifdef UNIX
			usleep(500000);
#endif
#ifdef __WIN32
			Sleep(500);
#endif
		}
	}
	else {
		printf("Connecting to %s:%d\n", inet_ntoa(engp->hostIP), engp->hostCtrlPort);
/*
		printf("active = %d, tcpconncomp = %d, state = %d, ctrlConnOK = %d\n",
			engp->active, engp->tcpCtrlConnectComplete,
			engp->state, ((WSInfo *)(engp->ctrlRefp))->ctrlConnOK);
*/
		if (StartClientContext(engp) != 0) {
			printf("StartClientContext() failed\n");
			exit(1);
		}
		while (engp->state != CLSM_FAILED && engp->state != CLSM_COMPLETE) {
			if (engp->state != state) {
				state = engp->state;
//				printf("Client state change: new state = %d\n", state);
			}
			RunClientContext(engp);
			if (engp->failCode != 0) {
				printf("Failcode: %lu\n", engp->failCode);
				printf("ioError:  %lu\n", engp->ioError);
			}

			// Release CPU occasionally
			if (iter++ % 20 == 0) {
#ifdef UNIX
				usleep(30000);
#endif
#ifdef __WIN32
				Sleep(30);
#endif
			}

//			if (engp->callMeAgain != 0) 
//				Sleep(engp->callMeAgain / 1000);
		}
	}
	ReportStats(engp);
}
	

void ReportStats(TPEngine *engp) {
	int throughput, i;
	LONG_LONG recvtime;
	struct tm *tmPnt;
	time_t stoptime;

		if (engp->tpMode == M_QUERY_MASTER) {
		for (i = 0; i < engp->numServers; i++) 
			printf("%02d:  \"%s\" (%s:%d)\n",
				i, engp->serverInfoList[i], engp->serverNameList[i], engp->serverPortList[i]);
	}
	else {
		printf("Test results:\n");
		printf("-------------\n");
		printf("Server: %s:%d\n", engp->hostName, engp->hostCtrlPort);
		printf("Test:   %ld\n", engp->tpMode);
		printf("Time:	%lu            Timelimit:    %lu\n",
			engp->sessionTime, engp->sessionMaxTime);
		time(&stoptime);
		printf("Test started: %s", ctime(&(engp->startTime)) );
		printf("Test ended:   %s", ctime(&stoptime));
		if (engp->tpMode == M_TCP_SEND || engp->tpMode == M_TCP_RECV) 
			printf("TCP Bytes: %lu\n", engp->tcpBytes);
		else {
			printf("# of packets: %lu\n", engp->nPackets);
			printf("Packetsize:   %lu\n", engp->packetSize);
		}

	  tmPnt = localtime( (time_t *)(&engp->stats.StartSend.tv_sec) );
	  printf("Send start: %04d-%02d-%02d %02d:%02d:%02d.%03ld\n",
		  tmPnt->tm_year + 1900, tmPnt->tm_mon + 1, tmPnt->tm_mday,
		  tmPnt->tm_hour, tmPnt->tm_min, tmPnt->tm_sec,
		  engp->stats.StartSend.tv_usec / 1000L );
	
	  tmPnt = localtime( (time_t *)(&engp->stats.StopSend.tv_sec) );
	    printf("Send stop : %04d-%02d-%02d %02d:%02d:%02d.%03ld\n",
		  tmPnt->tm_year + 1900, tmPnt->tm_mon + 1, tmPnt->tm_mday,
		  tmPnt->tm_hour, tmPnt->tm_min, tmPnt->tm_sec,
		  engp->stats.StopSend.tv_usec / 1000L );
	
	  tmPnt = localtime( (time_t *)(&engp->stats.StartRecv.tv_sec) );
	  printf("Recv start: %04d-%02d-%02d %02d:%02d:%02d.%03ld\n",
		  tmPnt->tm_year + 1900, tmPnt->tm_mon + 1, tmPnt->tm_mday,
		  tmPnt->tm_hour, tmPnt->tm_min, tmPnt->tm_sec,
		  engp->stats.StartRecv.tv_usec / 1000L );

	  tmPnt = localtime( (time_t *)(&engp->stats.StopRecv.tv_sec) );
	  printf("Recv stop : %04d-%02d-%02d %02d:%02d:%02d.%03ld\n",
		  tmPnt->tm_year + 1900, tmPnt->tm_mon + 1, tmPnt->tm_mday,
		  tmPnt->tm_hour, tmPnt->tm_min, tmPnt->tm_sec,
		  engp->stats.StopRecv.tv_usec / 1000L );

		if (engp->tpMode == M_TCP_SEND || engp->tpMode == M_TCP_RECV) {
		} else {
			printf("Packets sent:     %lu\n", engp->stats.PktsSent);
			printf("Packets received: %lu\n", engp->stats.PktsRecvd);
			printf("Packets lost:     %lu\n", engp->stats.PktsSent -
				engp->stats.PktsRecvd);
			printf("Packets unsent:   %lu\n", engp->stats.PktsUnSent);
			printf("OO Packets:       %lu\n", engp->stats.ooCount);
			if (engp->tpMode == M_UDP_FDX) {
				if (engp->stats.nRoundtrips > 0) {
					printf("Max roundtrip: %0.3fms\n", (double)engp->stats.MaxRoundtrip / 1000.0);
					printf("Min roundtrip: %0.3fms\n", (double)engp->stats.MinRoundtrip / 1000.0);
					printf("Avg roundtrip: %0.3fms\n", 
						((double)engp->stats.TotalRoundtrip / (double)engp->stats.nRoundtrips) / 1000.0);
				}
			}
		}
		printf("Bytes sent: %" LONG_LONG_PREFIX "d\n", engp->stats.BytesSent);
		printf("Bytes rcvd: %" LONG_LONG_PREFIX "d\n", engp->stats.BytesRecvd);
		recvtime = (LONG_LONG)engp->stats.StopRecv.tv_sec * (LONG_LONG)1000000 + 
				(LONG_LONG)engp->stats.StopRecv.tv_usec;
		recvtime -= (	(LONG_LONG)engp->stats.StartRecv.tv_sec * (LONG_LONG)1000000 + 
				(LONG_LONG)engp->stats.StartRecv.tv_usec );
		if (recvtime > 0) 
			throughput = (int)((double)(engp->stats.BytesRecvd * 8)/((double)recvtime / 1000000.0));
		else
			throughput = 0;
		printf("Throughput: %d bps (%s)\n\n\n", (int)throughput, Int32ToString((int)throughput));
	}
}
